# Trust Platform Design Suite - Usecase Help - AWS Connect

This document helps to understand Pre and Post steps of Usecase transaction diagram.

## Setup requirements
 - [EV76R77A](https://www.microchip.com/en-us/development-tool/EV76R77A)
 - [ATWINC1500-XPRO](https://www.microchip.com/en-us/development-tool/ATWINC1500-XPRO)
 - [MPLAB X IDE](https://www.microchip.com/en-us/development-tools-tools-and-software/mplab-x-ide) v6.00 or above

## Pre Usecase transaction Steps
 - Connect EV76R77A board to PC running Trust Platform Design Suite. It is required to connect both TARGET USB and DEBUG USB to PC.
 - ATWINC1500-XPRO should be connected to EXT3 of PIC32CMLS60 Curiosity PRO board
 - Ensure *MPLAB X Path* is set in *File* -> *Preference* under *System Settings*. This helps
    - To program the Usecase prototyping kit to factory reset application by TPDS
    - To open the embedded project of the Usecase
 - Setup AWS account. Follow the instructions at [**AWS demo account setup**](#aws-account-setup-instructions)
 - Ensure *~/.trustplatform/aws_credentials.yaml* contains the account credentials.
    - ~ indicates home directory.
        - Windows home directory is \user\username
        - Mac home directory is /users/username
        - Most Linux/Unix home directory is /home/username
 - Note that *~/.trustplatform/pic32cmls60_cloud_connect* is the **Usecase working directory**. It contains the resources generated during transaction diagram execution.
 - To update Winc Firmware on older versions of ATWINC1500-XPRO, please follow instructions [here](https://microchip-mplab-harmony.github.io/reference_apps/apps/sam_d21_iot/google_cloud_iot_core/utilities/readme.html)

## Post Usecase transaction Steps
On completing Usecase steps execution on TPDS, it is possible to either run the embedded project or view C source files by clicking *MPLAB X Project* or *C Source Folder* button.

- Once the Usecase project is loaded on MPLAB X IDE,
    - Set the project as Main -> right click on Project and select *Set as Main Project*
    - Set WiFi SSID and Password -> Open *cloud_wifi_config.h* under Project Header Files -> common -> cloud_wifi_config.h.
        - Uncomment and update WLAN_SSID and WLAN_PSK macros with user's wifi SSID and password
    - Set the configuration -> right click on Project, expand *Set Configuration* to select *AWS_CONNECT*
    - Build and Program the project -> right click on Project and select *Make and Program Device*
- Log from the embedded project can be viewed using applications like TeraTerm. Select the COM port and set baud rate as 115200-8-N-1

Example TeraTerm Log after successfully executing embedded project:

- Connecting to Cloud messages appear along with the led state.

![AWS Connect TeraTerm Log](images/aws_ttlog.png "AWS Connect TeraTerm Log")

## AWS Account Setup Instructions
In order to run the AWS Cloud Connect Usecases an AWS account is required. This document describes the steps required to obtain and configure an AWS account for the demo.

[Amazon Web Services (AWS)](https://aws.amazon.com/) provides computing services for a fee. Some are offered for free on a trial or small-scale basis. By signing up for your own AWS account, you are establishing an account to gain access to a wide range of computing services.

Think of your AWS account as your root account to AWS services. It is very powerful and gives you complete access. Be sure to protect your username and password. You control access to your AWS account by creating individual users and groups using the Identity and Access Management (IAM) Console. From the IAM Console, you also assign policies (permissions) to the group.

### Create your own AWS account
 1. Create AWS account
    - Go to https://aws.amazon.com/ and follow instructions to create your own AWS account. Additional details can be found at https://aws.amazon.com/premiumsupport/knowledge-center/create-and-activate-aws-account/.

 2. Secure root account with MFA (multi-factor authentication)

    This is an important step to better secure your root account against attackers. Anyone logging in not only needs to know the password, but also a constantly changing code generated by an MFA device.

    AWS recommends a number of MFA device options at the following link: https://aws.amazon.com/iam/details/mfa/

    The quickest solution is a virtual MFA device running on a phone. These apps provide the ability to scan the QR code AWS will generate to set up the MFA device.

    1. Return to https://aws.amazon.com/ and click the **Sign In to the Console**
    2. If it asks for an IAM user name and password, select the **Sign-in using root account credentials** link.
    3. Enter the email and password for your AWS account.
    4. Under **Find Services** search for **IAM** and select it to bring up the Identity and Access Management options.
    5. Click on **Activate MFA (Multi-factor Authentication) on your root account**

 3. Create an admin IAM user AWS best practices recommend not using your root account for standard administrative tasks, but to create a special admin user for those tasks. See https://docs.aws.amazon.com/IAM/latest/UserGuide/best-practices.html#lock-away-credentials

 4. Follow the instructions at https://docs.aws.amazon.com/IAM/latest/UserGuide/getting-started_create-admin-group.html for creating an admin user.

 5. Enable MFA (multi-factor authentication) for the admin user. See https://docs.aws.amazon.com/IAM/latest/UserGuide/best-practices.html#enable-mfa-for-privileged-users


### Configuring the account using CloudFormation Templates

The usage of a custom PKI with TrustFLEX devices uses the Just-In-Time Registration (JITR) feature of AWS IoT Core. This feature requires a number of resources setup with an AWS account to work. The creation of these resources is automated through the AWS CloudFormation service.

1. Sign into the AWS console (https://aws.amazon.com/) using the admin user created in the previous section.
2. Change to region to **US East (Ohio)** (a.k.a. us-east-2). This is done from a dropdown in the top right of the console webpage after logging in.
3. Under **Find Services** search for **CloudFormation** and select it to bring up that service.
4. Click **Create Stack** button.
5. Select **Upload a template file** from the page of the stack creation.
6. Click **Choose file** and upload the **aws-zero-touch-full-setup.yaml** file. Note, if running from a China region, you’ll need to select the **aws-zero-touch-full-setup-cn.yaml** instead. These files are available in ~/.trustplatform folder.
7. Click **Next** to move on to the stack details.
8. Enter **TrustFLEX** as the stack name. Actual name isn’t important, just has to be unique.
9. Enter a password for the user that will be created to run the demo under **UserPassword**.
10. Click **Next** to move on to the stack options. Nothing needs to be changed here.
11. Click **Next** to move on to the stack review.
12. Check the acknowledgement box regarding IAM resources at the bottom of the page.
13. Click **Create Stack** to start the resource creation.
14. Wait until the stack creation completes. This can take a few minutes. Once done, the stack your created will show as CREATE_COMPLETE.
15. Save demo credentials. Click the **Outputs** tab for the stack to see the credentials to be saved.

Save the credentials to aws_credentials.yaml file in ~/.trustplatform folder.